import type { INotebookContent } from '@jupyterlab/nbformat';
import { NotebookModel } from '@jupyterlab/notebook';
import { type IRenderMime } from '@jupyterlab/rendermime';
import type { Kernel, ServerConnection, ServiceManager } from '@jupyterlab/services';
import { CommandRegistry } from '@lumino/commands';
import type { OnSessionConnection } from '../../state';
import type { DatalayerNotebookExtension } from './Notebook';
/**
 * Base notebook component properties
 */
export interface IBaseNotebookProps {
    /**
     * Custom command registry.
     *
     * Note:
     * Providing it allows to command the component from an higher level.
     */
    commands?: CommandRegistry;
    /**
     * Notebook ID
     */
    id?: string;
    /**
     * Notebook extensions
     */
    extensions?: DatalayerNotebookExtension[];
    /**
     * Kernel ID to connect to
     */
    kernelId?: string;
    /**
     * Additional cell output renderers.
     */
    renderers?: IRenderMime.IRendererFactory[];
    /**
     * Service manager
     */
    serviceManager?: ServiceManager.IManager;
    /**
     * Notebook content model
     */
    model: NotebookModel;
    /**
     * Callback on session connection changed
     */
    onSessionConnectionChanged?: OnSessionConnection;
    /**
     * Notebook path
     *
     * Set this only if you use Jupyter Server to fetch
     * and save the content.
     */
    path?: string;
}
/**
 * Base notebook component
 *
 * The notebook model and kernel lifecycle is not handled by
 * this component.
 *
 * Important
 * This component is not connected to any React stores.
 */
export declare function BaseNotebook(props: IBaseNotebookProps): JSX.Element;
/**
 * Get the kernel ID to connect to.
 *
 * Steps:
 * - Check the requested kernel ID exists
 * - If no kernel found, start a new one if required
 *
 * Note
 * If the hook starts the kernel, it will shut it down when unmounting.
 */
export declare function useKernelId(options?: {
    /**
     * Kernels manager
     */
    kernels?: Kernel.IManager;
    /**
     * Kernel ID to connect to
     *
     * If the kernel does not exist and {@link startDefaultKernel} is `true`,
     * another kernel will be started.
     */
    requestedKernelId?: string;
    /**
     * Whether or not to start a default kernel.
     *
     * Default: false
     */
    startDefaultKernel?: boolean;
} | undefined): string | undefined;
export type CollaborationServer = {
    /**
     * Base server URL
     */
    baseURL: string;
    /**
     * Notebook room name to connect to.
     */
    roomName: string;
    /**
     * JWT token
     */
    token: string;
    /**
     * Server type
     */
    type: 'datalayer';
} | {
    /**
     * Notebook path
     */
    path: string;
    /**
     * Jupyter server settings
     */
    serverSettings: ServerConnection.ISettings;
    /**
     * Server type
     */
    type: 'jupyter';
};
/**
 * Hook to handle a notebook model.
 *
 * The notebook content may come from 3 sources:
 * - {@link nbformat}: The notebook content
 * - {@link url}: A URL to fetch the notebook content from
 * - {@link collaborationServer}: Parameters to connect to a collaboration server
 */
export declare function useNotebookModel(options: {
    /**
     * Collaboration server providing the document rooms
     */
    collaborationServer?: CollaborationServer;
    /**
     * Notebook content.
     */
    nbformat?: INotebookContent;
    /**
     * Whether the model is read-only or not.
     *
     * Default: false
     */
    readonly?: boolean;
    /**
     * URL to fetch the notebook content from.
     */
    url?: string;
}): NotebookModel | null;
