import { Fragment as _Fragment, jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
/*
 * Copyright (c) 2021-2023 Datalayer, Inc.
 *
 * MIT License
 */
import { useEffect, useMemo, useState } from 'react';
import { CodeMirrorEditorFactory, CodeMirrorMimeTypeService, EditorExtensionRegistry, EditorLanguageRegistry, EditorThemeRegistry, ybinding } from '@jupyterlab/codemirror';
import { Completer, CompleterModel, CompletionHandler, KernelCompleterProvider, ProviderReconciliator } from '@jupyterlab/completer';
import { PathExt, URLExt } from '@jupyterlab/coreutils';
import { Context } from '@jupyterlab/docregistry';
import { rendererFactory as javascriptRendererFactory } from '@jupyterlab/javascript-extension';
import { rendererFactory as jsonRendererFactory } from '@jupyterlab/json-extension';
import { createMarkdownParser } from '@jupyterlab/markedparser-extension';
import { MathJaxTypesetter } from '@jupyterlab/mathjax-extension';
import { NotebookModel, NotebookModelFactory, NotebookPanel, NotebookTracker, NotebookWidgetFactory, StaticNotebook } from '@jupyterlab/notebook';
import { RenderMimeRegistry, standardRendererFactories } from '@jupyterlab/rendermime';
import { YNotebook } from '@jupyter/ydoc';
import { find } from '@lumino/algorithm';
import { CommandRegistry } from '@lumino/commands';
import { PromiseDelegate } from '@lumino/coreutils';
import { DisposableSet } from '@lumino/disposable';
import { Signal } from '@lumino/signaling';
import { Widget } from '@lumino/widgets';
import { Box } from '@primer/react';
import { Banner } from '@primer/react/experimental';
import { EditorView } from 'codemirror';
import { WebsocketProvider } from 'y-websocket';
import { COLLABORATION_ROOM_URL_PATH, fetchSessionId, requestDocSession, WIDGET_MIMETYPE, WidgetLabRenderer, WidgetManager } from '../../jupyter';
import { newUuid, remoteUserCursors } from '../../utils';
import { Lumino } from '../lumino';
import { Loader } from '../utils';
import { addNotebookCommands } from './NotebookCommands';
const COMPLETER_TIMEOUT_MILLISECONDS = 1000;
const DEFAULT_EXTENSIONS = new Array();
const FALLBACK_NOTEBOOK_PATH = '.datalayer/ping.ipynb';
/**
 * Base notebook component
 *
 * The notebook model and kernel lifecycle is not handled by
 * this component.
 *
 * Important
 * This component is not connected to any React stores.
 */
export function BaseNotebook(props) {
    const { commands, extensions = DEFAULT_EXTENSIONS, kernelId, renderers, serviceManager, model, onSessionConnectionChanged, } = props;
    const [isLoading, setIsLoading] = useState(true);
    const [extensionComponents, setExtensionComponents] = useState(new Array());
    const id = useMemo(() => props.id || newUuid(), [props.id]);
    const path = useMemo(() => props.path || FALLBACK_NOTEBOOK_PATH, [props.path]);
    // Features
    const features = useMemo(() => new CommonFeatures({ commands, renderers }), [commands, renderers]);
    // Content factory
    const contentFactory = useMemo(() => new NotebookPanel.ContentFactory({
        editorFactory: features.editorServices.factoryService.newInlineEditor,
    }), [features.editorServices.factoryService.newInlineEditor]);
    // Completer
    const [completer, setCompleter] = useState(null);
    useEffect(() => {
        const completer = new Completer({ model: new CompleterModel() });
        // Dummy widget to initialize
        const widget = new Widget();
        const reconciliator = new ProviderReconciliator({
            context: {
                widget,
            },
            providers: [new KernelCompleterProvider()],
            timeout: COMPLETER_TIMEOUT_MILLISECONDS,
        });
        const handler = new CompletionHandler({ completer, reconciliator });
        completer.hide();
        Widget.attach(completer, document.body);
        setCompleter(handler);
        return () => {
            handler.dispose();
            completer.dispose();
            widget.dispose();
        };
    }, []);
    // Widget factory
    const [widgetFactory, setWidgetFactory] = useState(null);
    useEffect(() => {
        const thisFactory = new NotebookWidgetFactory({
            name: 'Notebook',
            label: 'Notebook',
            modelName: 'notebook',
            fileTypes: ['notebook'],
            defaultFor: ['notebook'],
            preferKernel: true,
            autoStartDefault: false,
            canStartKernel: false,
            shutdownOnClose: false,
            rendermime: features.rendermime,
            contentFactory,
            mimeTypeService: features.editorServices.mimeTypeService,
            notebookConfig: {
                ...StaticNotebook.defaultNotebookConfig,
                recordTiming: true,
            },
        });
        setWidgetFactory(thisFactory);
        return () => {
            thisFactory.dispose();
            setWidgetFactory(factory => (factory === thisFactory ? null : factory));
        };
    }, [contentFactory, features]);
    // Tracker & commands
    const [tracker, setTracker] = useState(null);
    useEffect(() => {
        const thisTracker = completer
            ? new NotebookTracker({ namespace: id })
            : null;
        const commands = completer
            ? addNotebookCommands(features.commands, completer, thisTracker, props.path)
            : null;
        setTracker(thisTracker);
        return () => {
            commands?.dispose();
            thisTracker?.dispose();
            setTracker(tracker => (tracker === thisTracker ? null : tracker));
        };
    }, [completer, id, features.commands, props.path]);
    // Context
    const [context, setContext] = useState(null);
    useEffect(() => {
        const factory = new DummyModelFactory(model);
        const thisContext = new Context({
            factory,
            manager: serviceManager ?? new NoServiceManager(),
            path,
            kernelPreference: {
                shouldStart: false,
                canStart: false,
                autoStartDefault: false,
                shutdownOnDispose: false,
            },
        });
        initializeContext(thisContext, id, 
        // Initialization must not trigger revert in case we set up the model content
        path !== FALLBACK_NOTEBOOK_PATH ? path : undefined, onSessionConnectionChanged, !serviceManager);
        setContext(thisContext);
        return () => {
            thisContext.dispose();
            factory.dispose();
            setContext(context => (context === thisContext ? null : context));
        };
    }, [id, serviceManager, model, onSessionConnectionChanged, path]);
    // Set kernel
    useEffect(() => {
        if (context && kernelId && !context.sessionContext.isDisposed) {
            context.sessionContext.changeKernel({ id: kernelId }).catch(reason => {
                console.error('Failed to change kernel model.', reason);
            });
        }
    }, [context, kernelId]);
    // Notebook
    const [panel, setPanel] = useState(null);
    useEffect(() => {
        let thisPanel = null;
        let widgetsManager = null;
        if (context) {
            thisPanel = widgetFactory?.createNew(context) ?? null;
            if (thisPanel) {
                setExtensionComponents(extensions.map(extension => {
                    extension.init({
                        notebookId: id,
                        commands: features.commands,
                        panel: thisPanel,
                    });
                    extension.createNew(thisPanel, context);
                    return extension.component ?? _jsx(_Fragment, {});
                }));
                //-- Add ipywidgets renderer
                const notebookRenderers = thisPanel.content.rendermime;
                widgetsManager = new WidgetManager(context, notebookRenderers, {
                    saveState: false,
                });
                notebookRenderers.addFactory({
                    safe: true,
                    mimeTypes: [WIDGET_MIMETYPE],
                    defaultRank: 1,
                    createRenderer: options => new WidgetLabRenderer(options, widgetsManager),
                }, 1);
                for (const cell of thisPanel.content.widgets) {
                    if (cell.model.type === 'code') {
                        for (const codecell of cell.outputArea.widgets) {
                            for (const output of Array.from(codecell.children())) {
                                if (output instanceof WidgetLabRenderer) {
                                    output.manager = widgetsManager;
                                }
                            }
                        }
                    }
                }
                setIsLoading(false);
            }
        }
        setPanel(thisPanel);
        if (!thisPanel) {
            setExtensionComponents([]);
        }
        return () => {
            widgetsManager?.dispose();
            if (thisPanel) {
                if (thisPanel.content) {
                    Signal.clearData(thisPanel.content);
                }
                try {
                    thisPanel.dispose();
                }
                catch (reason) {
                    // No-op
                }
            }
            setPanel(panel => (panel === thisPanel ? null : panel));
        };
    }, [context, extensions, features.commands, widgetFactory]);
    useEffect(() => {
        let isMounted = true;
        let onActiveCellChanged = null;
        let onSessionChanged = null;
        if (panel) {
            if (tracker) {
                if (!tracker.has(panel)) {
                    tracker.add(panel).catch(reason => {
                        console.error(`Failed to track the notebook panel '${id}'.`);
                    });
                }
            }
            if (completer) {
                // Setup the completer here as it requires the panel
                onActiveCellChanged = (notebook, cell) => {
                    if (cell) {
                        cell.ready.then(() => {
                            completer.editor = cell?.editor;
                        });
                    }
                };
                onSessionChanged = (_, changes) => {
                    if (!isMounted) {
                        return;
                    }
                    const editor = panel.content.activeCell?.editor;
                    const provider = new KernelCompleterProvider();
                    const reconciliator = new ProviderReconciliator({
                        context: {
                            widget: panel,
                            editor,
                            session: changes.newValue,
                        },
                        providers: [provider],
                        timeout: COMPLETER_TIMEOUT_MILLISECONDS,
                    });
                    completer.editor = editor;
                    completer.reconciliator = reconciliator;
                };
                panel.context.sessionContext.sessionChanged.connect(onSessionChanged);
                panel.context.sessionContext.ready.then(() => {
                    onSessionChanged?.(panel.context.sessionContext, {
                        name: 'session',
                        oldValue: null,
                        newValue: panel.context.sessionContext.session,
                    });
                });
                panel.content.activeCellChanged.connect(onActiveCellChanged);
            }
        }
        return () => {
            isMounted = false;
            // Reset the completer
            if (completer) {
                if (onActiveCellChanged) {
                    panel?.content.activeCellChanged.disconnect(onActiveCellChanged);
                }
                if (onSessionChanged) {
                    panel?.context.sessionContext.sessionChanged.connect(onSessionChanged);
                }
                completer.editor = null;
                completer.reconciliator = new ProviderReconciliator({
                    context: {
                        widget: new Widget(),
                    },
                    providers: [new KernelCompleterProvider()],
                    timeout: COMPLETER_TIMEOUT_MILLISECONDS,
                });
            }
        };
    }, [completer, panel, tracker]);
    useEffect(() => {
        const onKeyDown = (event) => {
            features.commands.processKeydownEvent(event);
        };
        // FIXME It would be better to add the listener to the Box wrapping the panel
        // but this requires the use of the latest version of JupyterLab/Lumino that
        // capture event at bubbling phase for keyboard shortcuts rather than at capture phase.
        document.addEventListener('keydown', onKeyDown, true);
        return () => {
            document.removeEventListener('keydown', onKeyDown, true);
        };
    }, [features.commands]);
    return (_jsxs(_Fragment, { children: [extensionComponents.map((extensionComponent, index) => {
                return (_jsx(Box, { children: extensionComponent }, `${extensionComponent}-${index}`));
            }), isLoading ? (_jsx(Loader, {}, "notebook-loader")) : panel ? (_jsx(Box, { sx: { height: '100%' }, children: _jsx(Lumino, { id: id, children: panel }, "notebook-container") })) : (_jsx(Banner, { variant: "critical", description: "Unable to create the notebook view.", hideTitle: true, title: "Error" }, "notebook-error"))] }));
}
/**
 * Get the kernel ID to connect to.
 *
 * Steps:
 * - Check the requested kernel ID exists
 * - If no kernel found, start a new one if required
 *
 * Note
 * If the hook starts the kernel, it will shut it down when unmounting.
 */
export function useKernelId(options = {}) {
    const { kernels, requestedKernelId, startDefaultKernel = false } = options;
    // Define the kernel to be used.
    // - Check the provided kernel id exists
    // - If no kernel found, start a new one if required
    const [kernelId, setKernelId] = useState(undefined);
    useEffect(() => {
        let isMounted = true;
        let connection;
        if (kernels) {
            (async () => {
                let newKernelId;
                await kernels.ready;
                if (requestedKernelId) {
                    for (const model of kernels.running()) {
                        if (model.id === requestedKernelId) {
                            newKernelId = requestedKernelId;
                            break;
                        }
                    }
                }
                if (!newKernelId && startDefaultKernel && isMounted) {
                    console.log('Starting new kernel.');
                    connection = await kernels.startNew();
                    if (isMounted) {
                        newKernelId = connection.id;
                    }
                    else {
                        connection.dispose();
                    }
                }
                if (isMounted) {
                    setKernelId(newKernelId);
                }
            })();
        }
        return () => {
            isMounted = false;
            if (connection) {
                // A new kernel was started
                console.log(`Shutting down kernel '${connection.id}'.`);
                connection
                    .shutdown()
                    .catch(reason => {
                    console.warn(`Failed to shutdown kernel '${connection?.id}'.`, reason);
                })
                    .finally(() => {
                    connection?.dispose();
                });
            }
        };
    }, [kernels, requestedKernelId, startDefaultKernel]);
    return kernelId;
}
/**
 * Hook to handle a notebook model.
 *
 * The notebook content may come from 3 sources:
 * - {@link nbformat}: The notebook content
 * - {@link url}: A URL to fetch the notebook content from
 * - {@link collaborationServer}: Parameters to connect to a collaboration server
 */
export function useNotebookModel(options) {
    const { collaborationServer, nbformat, readonly = false, url } = options;
    // Generate the notebook model
    // There are three posibilities (by priority order):
    // - Connection to a collaborative room
    // - Provided notebook content
    // - Provided URL to fetch notebook content from
    const [model, setModel] = useState(null);
    useEffect(() => {
        let isMounted = true;
        const disposable = new DisposableSet();
        if (collaborationServer) {
            // As the server has the content source of thruth, we
            // must ensure that the shared model is pristine before
            // to connect to the server. More over we should ensure,
            // the connection is disposed in case the server room is
            // reset for any reason while the client is still alive.
            let provider = null;
            let ready = new PromiseDelegate();
            const isMounted = true;
            let sharedModel = null;
            const onConnectionClose = (event) => {
                if (event.code > 1000) {
                    console.error('Connection with the room has been closed unexpectedly.', event);
                    provider?.disconnect();
                    // If sessionId has expired - reset the client model
                    if (event.code === 4002) {
                        disposable.clear();
                        provider?.destroy();
                        if (isMounted) {
                            connect().catch(error => {
                                console.error('Failed to setup collaboration connection.', error);
                            });
                        }
                    }
                    // FIXME inform the user.
                }
            };
            const onSync = (isSynced) => {
                if (isSynced) {
                    provider?.off('sync', onSync);
                    ready.resolve(void 0);
                }
            };
            const connect = async () => {
                ready.reject('Connection closed.');
                ready = new PromiseDelegate();
                sharedModel = new YNotebook();
                const { ydoc, awareness } = sharedModel;
                let roomURL = '';
                let roomName = '';
                const params = {};
                // Setup Collaboration
                if (collaborationServer.type === 'jupyter') {
                    const { path, serverSettings } = collaborationServer;
                    const session = await requestDocSession('json', 'notebook', path, serverSettings);
                    roomURL = URLExt.join(serverSettings.wsUrl, COLLABORATION_ROOM_URL_PATH);
                    roomName = `${session.format}:${session.type}:${session.fileId}`;
                    params.sessionId = session.sessionId;
                    if (serverSettings.token) {
                        params.token = serverSettings.token;
                    }
                }
                else if (collaborationServer.type === 'datalayer') {
                    const { baseURL, roomName: roomName_, token } = collaborationServer;
                    roomName = roomName_; // Set non local variable
                    const serverURL = URLExt.join(baseURL, '/api/spacer/v1/rooms');
                    roomURL = serverURL.replace(/^http/, 'ws');
                    params.sessionId = await fetchSessionId({
                        url: URLExt.join(serverURL, roomName),
                        token,
                    });
                    params.token = token;
                }
                if (params.sessionId) {
                    provider = new WebsocketProvider(roomURL, roomName, ydoc, {
                        disableBc: true,
                        params,
                        awareness,
                    });
                    provider.on('sync', onSync);
                    provider.on('connection-close', onConnectionClose);
                    console.log('Collaboration is setup with websocket provider.');
                    await ready.promise;
                    const dispose = () => {
                        provider?.off('sync', onSync);
                        provider?.off('connection-close', onConnectionClose);
                        provider?.disconnect();
                        provider?.destroy();
                    };
                    if (isMounted) {
                        // Create a new model using the one synchronized with the collaboration room
                        const model = new NotebookModel({
                            collaborationEnabled: true,
                            disableDocumentWideUndoRedo: true,
                            sharedModel,
                        });
                        model.readOnly = readonly;
                        setModel(model);
                        disposable.add(Object.freeze({ dispose, isDisposed: false }));
                    }
                    else {
                        dispose();
                    }
                }
            };
            connect().catch(error => {
                console.error('Failed to setup collaboration connection.', error);
            });
        }
        else {
            const createModel = (nbformat) => {
                const model = new NotebookModel();
                if (nbformat) {
                    nbformat.cells.forEach(cell => {
                        cell.metadata['editable'] = !readonly;
                    });
                    model.fromJSON(nbformat);
                }
                model.readOnly = readonly;
                setModel(model);
            };
            if (!nbformat && url) {
                loadFromUrl(url).then(nbformat => {
                    if (isMounted) {
                        createModel(nbformat);
                    }
                });
            }
            else {
                createModel(nbformat);
            }
        }
        return () => {
            isMounted = false;
            disposable.dispose();
        };
    }, [collaborationServer, nbformat, readonly, url]);
    return model;
}
async function loadFromUrl(url) {
    return fetch(url)
        .then(response => {
        return response.text();
    })
        .then(nb => {
        return JSON.parse(nb);
    });
}
/**
 * Common immutable JupyterLab features required for the notebook panel.
 */
class CommonFeatures {
    _commands;
    _editorServices;
    _rendermime;
    constructor(options = {}) {
        this._commands = options.commands ?? new CommandRegistry();
        const languages = new EditorLanguageRegistry();
        // Register default languages.
        for (const language of EditorLanguageRegistry.getDefaultLanguages()) {
            languages.addLanguage(language);
        }
        // Add Jupyter Markdown flavor here to support code block highlighting.
        languages.addLanguage({
            name: 'ipythongfm',
            mime: 'text/x-ipythongfm',
            load: async () => {
                // TODO: add support for LaTeX.
                const m = await import('@codemirror/lang-markdown');
                return m.markdown({
                    codeLanguages: (info) => languages.findBest(info),
                });
            },
        });
        const initialFactories = standardRendererFactories
            .filter(factory => factory.mimeTypes[0] !== 'text/javascript')
            .concat([jsonRendererFactory, javascriptRendererFactory])
            .concat(options.renderers ?? []);
        this._rendermime = new RenderMimeRegistry({
            initialFactories,
            latexTypesetter: new MathJaxTypesetter(),
            markdownParser: createMarkdownParser(languages),
        });
        const mimeTypeService = new CodeMirrorMimeTypeService(languages);
        const themes = new EditorThemeRegistry();
        for (const theme of EditorThemeRegistry.getDefaultThemes()) {
            themes.addTheme(theme);
        }
        const editorExtensions = new EditorExtensionRegistry();
        for (const extensionFactory of EditorExtensionRegistry.getDefaultExtensions({ themes })) {
            editorExtensions.addExtension(extensionFactory);
        }
        editorExtensions.addExtension({
            name: 'shared-model-binding',
            factory: options => {
                const sharedModel = options.model.sharedModel;
                return EditorExtensionRegistry.createImmutableExtension(ybinding({
                    ytext: sharedModel.ysource,
                    undoManager: sharedModel.undoManager ?? undefined,
                }));
            },
        });
        editorExtensions.addExtension({
            name: 'remote-user-cursors',
            factory(options) {
                const { awareness, ysource: ytext } = options.model.sharedModel;
                return EditorExtensionRegistry.createImmutableExtension(remoteUserCursors({ awareness, ytext }));
            },
        });
        // Fix to deal with Content Security Policy
        const nonce = getNonce();
        if (nonce) {
            editorExtensions.addExtension({
                name: 'csp-nonce',
                factory() {
                    return EditorExtensionRegistry.createImmutableExtension(EditorView.cspNonce.of(nonce));
                },
            });
        }
        const factoryService = new CodeMirrorEditorFactory({
            extensions: editorExtensions,
            languages: languages,
        });
        this._editorServices = {
            factoryService,
            mimeTypeService,
        };
    }
    get commands() {
        return this._commands;
    }
    get editorServices() {
        return this._editorServices;
    }
    get rendermime() {
        return this._rendermime;
    }
}
/**
 * Dummy notebook factory
 *
 * It returns the provided notebook model.
 */
class DummyModelFactory extends NotebookModelFactory {
    model;
    constructor(model) {
        super();
        this.model = model;
    }
    createNew(options) {
        return this.model;
    }
}
function initializeContext(context, id, path, onSessionConnection, serverLess = false) {
    const shuntContentManager = path ? false : true;
    // TODO we should implement our own thing rather this ugly Javascript patch.
    // These are fixes on the Context and the SessionContext to have more control on the kernel launch.
    context.sessionContext._initialize = async () => {
        const manager = context.sessionContext.sessionManager;
        await manager.ready;
        await manager.refreshRunning();
        const model = find(manager.running(), model => {
            // !! we need to set the kernelPreference id
            return (model.kernel?.id === (context.sessionContext.kernelPreference.id ?? ''));
        });
        if (model) {
            try {
                const session = manager.connectTo({
                    model: {
                        ...model,
                        path: path ?? model.path,
                        name: path ?? model.name,
                    },
                    kernelConnectionOptions: {
                        handleComms: true,
                    },
                });
                context.sessionContext._handleNewSession(session);
                // Dispose the previous KernelConnection to avoid errors with Comms.
                this._kernel?.connection?.dispose();
            }
            catch (err) {
                void context.sessionContext._handleSessionError(err);
                return Promise.reject(err);
            }
        }
        return await context.sessionContext._startIfNecessary();
    };
    // Custom dispose that does not dispose the model
    context.dispose = () => {
        if (context.isDisposed) {
            return;
        }
        context._isDisposed = true;
        context.sessionContext.dispose();
        context._disposed.emit(void 0);
        Signal.clearData(this);
    };
    if (shuntContentManager) {
        // If nbformat is provided and we don't want to interact with the Content Manager.
        context._populate = async () => {
            context._isPopulated = true;
            context._isReady = true;
            context._populatedPromise.resolve(void 0);
            // Add a checkpoint if none exists and the file is writable.
            // Force skip this step for nbformat notebooks.
            // await (context as any)._maybeCheckpoint(false);
            if (context.isDisposed) {
                return;
            }
            // Update the kernel preference.
            const name = context._model.defaultKernelName ||
                context.sessionContext.kernelPreference.name;
            context.sessionContext.kernelPreference = {
                ...context.sessionContext.kernelPreference,
                name,
                language: context._model.defaultKernelLanguage,
            };
            if (!serverLess) {
                // Note: we don't wait on the session to initialize
                // so that the user can be shown the content before
                // any kernel has started.
                void context.sessionContext
                    .initialize()
                    .then((shouldSelect) => {
                    if (shouldSelect) {
                        void context._dialogs.selectKernel(context.sessionContext.sessionContext);
                    }
                });
            }
        };
        context.initialize = async (isNew) => {
            context.model.dirty = false;
            const now = new Date().toISOString();
            const model = {
                path: id,
                name: id,
                type: 'notebook',
                content: undefined,
                writable: true,
                created: now,
                last_modified: now,
                mimetype: 'application/x-ipynb+json',
                format: 'json',
            };
            context._updateContentsModel(model);
            await context._populate();
            context.model.sharedModel.clearUndoHistory();
        };
    }
    // Connect signals
    context.sessionContext.sessionChanged.connect((_, args) => {
        const session = args.newValue;
        console.log('Current Jupyter Session Connection.', session);
        onSessionConnection?.(session ?? undefined);
    });
    context.sessionContext.kernelChanged.connect((_, args) => {
        const kernelConnection = args.newValue;
        console.log('Current Jupyter Kernel Connection.', kernelConnection);
        if (kernelConnection && !kernelConnection.handleComms) {
            console.log('Updating the current Kernel Connection to enforce Comms support.', kernelConnection.handleComms);
            kernelConnection.handleComms = true;
        }
    });
    context.sessionContext.ready.then(() => {
        onSessionConnection?.(context?.sessionContext.session ?? undefined);
    });
    // Initialize the context
    context.initialize(path ? false : true);
}
/**
 * Service manager facade for missing manager.
 */
class NoServiceManager {
    contents = Object.freeze({
        fileChanged: { connect: () => { } },
        getSharedModelFactory(path) {
            return null;
        },
        localPath(path) {
            const parts = path.split('/');
            const firstParts = parts[0].split(':');
            if (firstParts.length === 1 ||
                !this._additionalDrives.has(firstParts[0])) {
                return PathExt.removeSlash(path);
            }
            return PathExt.join(firstParts.slice(1).join(':'), ...parts.slice(1));
        },
        normalize(path) {
            const parts = path.split(':');
            if (parts.length === 1) {
                return PathExt.normalize(path);
            }
            return `${parts[0]}:${PathExt.normalize(parts.slice(1).join(':'))}`;
        },
    });
    kernelspecs = Object.freeze({
        // Ever hanging promise.
        ready: new Promise(() => { }),
    });
    sessions = Object.freeze({
        // Ever hanging promise.
        ready: new Promise(() => { }),
    });
    // Always ready otherwise the spinner in the MainAreaWidget won't be discarded.
    ready = Promise.resolve();
}
/**
 * Returns the nonce used in the page, if any.
 *
 * Based on https://github.com/cssinjs/jss/blob/master/packages/jss/src/DomRenderer.js
 */
function getNonce() {
    const node = document.querySelector('meta[property="csp-nonce"]');
    if (node) {
        return node.getAttribute('content');
    }
    else {
        return null;
    }
}
//# sourceMappingURL=NotebookBase.js.map