/*
 * Copyright (c) 2021-2023 Datalayer, Inc.
 *
 * MIT License
 */
import { createStore } from 'zustand/vanilla';
import { useStore } from 'zustand';
import { NotebookCommandIds } from './NotebookCommands';
export const notebookStore = createStore((set, get) => ({
    notebooks: new Map(),
    setNotebooks: (notebooks) => set((state) => ({ notebooks })),
    selectNotebook: (id) => {
        return get().notebooks.get(id);
    },
    selectNotebookAdapter: (id) => {
        const notebook = get().notebooks.get(id);
        if (notebook) {
            return notebook.adapter;
        }
        return undefined;
    },
    selectNotebookModel: (id) => {
        if (get().notebooks.get(id)) {
            return {
                model: get().notebooks.get(id)?.model,
                changed: get().notebooks.get(id)?.model?.contentChanged,
            };
        }
        return undefined;
    },
    selectKernelStatus: (id) => {
        return get().notebooks.get(id)?.kernelStatus;
    },
    selectActiveCell: (id) => {
        return get().notebooks.get(id)?.activeCell;
    },
    selectNotebookPortals: (id) => {
        return get().notebooks.get(id)?.portals;
    },
    selectSaveRequest: (id) => {
        return get().notebooks.get(id)?.saveRequest;
    },
    selectNotebookPortalDisplay: (id) => {
        return get().notebooks.get(id)?.portalDisplay;
    },
    run: (id) => { get().notebooks.get(id)?.adapter?.commands.execute(NotebookCommandIds.run); },
    runAll: (id) => { get().notebooks.get(id)?.adapter?.commands.execute(NotebookCommandIds.runAll); },
    interrupt: (id) => { get().notebooks.get(id)?.adapter?.commands.execute(NotebookCommandIds.interrupt); },
    insertAbove: (mutation) => {
        get().notebooks.get(mutation.id)?.adapter?.setDefaultCellType(mutation.cellType);
        get().notebooks.get(mutation.id)?.adapter?.insertAbove(mutation.source);
    },
    insertBelow: (mutation) => {
        get().notebooks.get(mutation.id)?.adapter?.setDefaultCellType(mutation.cellType);
        get().notebooks.get(mutation.id)?.adapter?.insertBelow(mutation.source);
    },
    delete: (id) => { get().notebooks.get(id)?.adapter?.commands.execute(NotebookCommandIds.deleteCells); },
    changeCellType: (mutation) => {
        get().notebooks.get(mutation.id)?.adapter?.changeCellType(mutation.cellType);
    },
    save: (mutation) => {
        get().notebooks.get(mutation.id)?.adapter?.commands.execute(NotebookCommandIds.save);
        const notebooks = get().notebooks;
        const notebook = notebooks.get(mutation.id);
        if (notebook) {
            notebook.saveRequest = mutation.date;
            set((state) => ({ notebooks }));
        }
    },
    reset: () => set((state) => ({ notebooks: new Map() })),
    update: (update) => {
        const notebooks = get().notebooks;
        let notebook = notebooks.get(update.id);
        if (notebook) {
            /*
            notebook = {
              ...notebook,
              ...update.state,
            };
            */
            // TODO Revisit this...
            notebook.adapter = update.state.adapter;
        }
        else {
            notebooks.set(update.id, {
                adapter: update.state.adapter,
                portals: [],
            });
        }
        set((state) => ({ notebooks }));
    },
    activeCellChange: (cellModelId) => {
        const notebooks = get().notebooks;
        const notebook = notebooks.get(cellModelId.id);
        if (notebook) {
            notebook.activeCell = cellModelId.cellModel;
            set((state) => ({ notebooks }));
        }
    },
    changeModel: (notebookModelId) => {
        const notebooks = get().notebooks;
        const notebook = notebooks.get(notebookModelId.id);
        if (notebook) {
            notebook.model = notebookModelId.notebookModel;
            set((state) => ({ notebooks }));
        }
    },
    changeNotebook: (notebookChangeId) => {
        const notebooks = get().notebooks;
        const notebook = notebooks.get(notebookChangeId.id);
        if (notebook) {
            notebook.notebookChange = notebookChangeId.notebookChange;
            set((state) => ({ notebooks }));
        }
    },
    changeKernel: (kernelChange) => {
        const notebooks = get().notebooks;
        const notebook = notebooks.get(kernelChange.id);
        if (notebook) {
            notebook.adapter?.assignKernel(kernelChange.kernel);
            set((state) => ({ notebooks }));
        }
    },
    changeKernelStatus: (kernelStatusId) => {
        const notebooks = get().notebooks;
        const notebook = notebooks.get(kernelStatusId.id);
        if (notebook) {
            notebook.kernelStatus = kernelStatusId.kernelStatus;
            set((state) => ({ notebooks }));
        }
    },
    addPortals: (portalsMutation) => {
        const notebooks = get().notebooks;
        const notebook = notebooks.get(portalsMutation.id);
        if (notebook) {
            notebook.portals = notebook.portals.concat(portalsMutation.portals);
            set((state) => ({ notebooks }));
        }
    },
    dispose: (id) => {
        get().setPortalDisplay({ id, portalDisplay: undefined });
        const notebooks = get().notebooks;
        const notebook = notebooks.get(id);
        if (notebook) {
            notebooks.delete(id);
        }
        set((state) => ({ notebooks }));
    },
    setPortals: (portalsMutation) => {
        const notebooks = get().notebooks;
        const notebook = notebooks.get(portalsMutation.id);
        if (notebook) {
            notebook.portals = portalsMutation.portals;
            set((state) => ({ notebooks }));
        }
    },
    setPortalDisplay: (portalDisplayMutation) => {
        const notebooks = get().notebooks;
        const notebook = notebooks.get(portalDisplayMutation.id);
        if (notebook) {
            notebook.portalDisplay = portalDisplayMutation.portalDisplay;
            set((state) => ({ notebooks }));
        }
    },
}));
export function useNotebookStore(selector) {
    return useStore(notebookStore, selector);
}
export default useNotebookStore;
//# sourceMappingURL=NotebookState.js.map