import { jsx as _jsx } from "react/jsx-runtime";
/*
 * Copyright (c) 2021-2023 Datalayer, Inc.
 *
 * MIT License
 */
import { ReactWidget } from '@jupyterlab/apputils';
import { Signal } from '@lumino/signaling';
import { JupyterReactTheme } from '../../../../theme';
import { CellSidebar } from './CellSidebar';
class CellSidebarFactory {
    panel;
    commands;
    nbgrader;
    sidebarWidth;
    factory;
    _isDisposed = false;
    _sidebars = new WeakMap();
    /**
     * Cell sidebar factory.
     *
     * @param panel The notebook panel
     * @param commands Command registry
     * @param ngrader Whether to activate nbgrader feature or not.
     */
    constructor(panel, commands, nbgrader = false, sidebarWidth = 120, factory = CellSidebar) {
        this.panel = panel;
        this.commands = commands;
        this.nbgrader = nbgrader;
        this.sidebarWidth = sidebarWidth;
        this.factory = factory;
        this._onModelChanged(panel.content);
        panel.content.modelChanged.connect(this._onModelChanged, this);
    }
    get isDisposed() {
        return this._isDisposed;
    }
    dispose() {
        if (this._isDisposed) {
            return;
        }
        this._isDisposed = true;
        this.panel.content.model?.cells.changed.disconnect(this._onCellsChanged, this);
        this.panel.content.modelChanged.disconnect(this._onModelChanged, this);
        Signal.clearData(this);
    }
    _addSidebar(model) {
        const cell = this._getCell(model);
        if (cell) {
            const Element = this.factory;
            const sidebar = ReactWidget.create(_jsx(JupyterReactTheme, { children: _jsx(Element, { commands: this.commands, model: model, nbgrader: this.nbgrader }) }));
            // Position sidebar wrapper
            sidebar.node.style.width = `${this.sidebarWidth}px`;
            sidebar.node.style.position = 'absolute';
            sidebar.node.style.right = `-${this.sidebarWidth}px`;
            sidebar.node.style.top = '0';
            cell.layout.insertWidget(0, sidebar);
            this._sidebars.set(model, sidebar);
            const removeSidebar = () => {
                this._sidebars.delete(model);
                sidebar.dispose();
            };
            cell.disposed.connect(removeSidebar);
        }
    }
    _getCell(model) {
        return this.panel?.content.widgets.find(widget => widget.model === model);
    }
    _removeSidebar(model) {
        this._sidebars.get(model)?.dispose();
        this._sidebars.delete(model);
    }
    _onCellsChanged(cells, changed) {
        changed.oldValues.forEach(model => this._removeSidebar(model));
        changed.newValues.forEach(model => this._addSidebar(model));
    }
    _onModelChanged(content) {
        const cells = content.model?.cells;
        if (cells) {
            this._onCellsChanged(cells, {
                type: 'add',
                newIndex: 0,
                newValues: Array.from(cells),
                oldIndex: -1,
                oldValues: [],
            });
        }
        cells?.changed.connect(this._onCellsChanged, this);
    }
}
/**
 * Cell sidebar extension for notebook panels.
 */
export class CellSidebarExtension {
    factory;
    commands;
    nbgrader;
    sidebarWidth;
    /**
     * Constructor
     *
     * @param commands Command registry
     * @param factory Cell sidebar React component factory
     * @param nbgrader Whether to activate nbgrader feature or not.
     */
    constructor(options = {}) {
        this.factory = options.factory ?? CellSidebar;
        this.commands = options.commands;
        this.nbgrader = options.nbgrader;
        this.sidebarWidth = options.sidebarWidth;
    }
    component;
    createNew(panel) {
        // We assume the extension was either created within JupyterLab passing
        // the app commands registry or through Datalayer workflow that set it
        // when calling `init`.
        const sidebar = new CellSidebarFactory(panel, this.commands, this.nbgrader, this.sidebarWidth, this.factory);
        return sidebar;
    }
    init(props) {
        this.commands = props.commands;
    }
}
export default CellSidebarExtension;
//# sourceMappingURL=CellSidebarExtension.js.map