/*
 * Copyright (c) 2021-2023 Datalayer, Inc.
 *
 * MIT License
 */
import { rendererFactory as javascriptRendererFactory } from '@jupyterlab/javascript-extension';
import { rendererFactory as jsonRendererFactory } from '@jupyterlab/json-extension';
import { OutputArea, OutputAreaModel, } from '@jupyterlab/outputarea';
import { RenderMimeRegistry, standardRendererFactories, } from '@jupyterlab/rendermime';
import { ClassicWidgetManager, WidgetRenderer, WIDGET_MIMETYPE } from '../../jupyter/ipywidgets/classic';
import { requireLoader as loader } from '../../jupyter/ipywidgets/libembed-amd';
import { execute } from './OutputExecutor';
export class OutputAdapter {
    _id;
    _kernel;
    _renderers;
    _outputArea;
    _rendermime;
    _iPyWidgetsManager;
    _suppressCodeExecutionErrors;
    constructor(id, kernel, outputs, outputAreaModel, suppressCodeExecutionErrors = false) {
        this._id = id;
        this._kernel = kernel;
        this._suppressCodeExecutionErrors = suppressCodeExecutionErrors;
        this._renderers = standardRendererFactories.filter(factory => factory.mimeTypes[0] !== 'text/javascript');
        this._renderers.push(jsonRendererFactory);
        this._renderers.push(javascriptRendererFactory);
        this._rendermime = new RenderMimeRegistry({
            initialFactories: this._renderers,
        });
        this._iPyWidgetsManager = new ClassicWidgetManager({ loader });
        this._rendermime.addFactory({
            safe: false,
            mimeTypes: [WIDGET_MIMETYPE],
            createRenderer: (options) => new WidgetRenderer(options, this._iPyWidgetsManager),
        }, 0);
        // const widgetRegistry = activateWidgetExtension(this._rendermime);
        // activatePlotlyWidgetExtension(widgetRegistry);
        const model = outputAreaModel ??
            new OutputAreaModel({
                trusted: true,
                values: outputs,
            });
        this._outputArea = new OutputArea({
            model,
            rendermime: this._rendermime,
        });
        if (outputs && outputs[0]) {
            const data = outputs[0].data;
            if (data) {
                const isPlotly = data['application/vnd.plotly.v1+json'];
                if (isPlotly) {
                    let script = this._outputArea.node.children[0].children[1].children[0]
                        .children[1].innerHTML;
                    script = script.replaceAll('\n,', '\n');
                    eval(script);
                }
            }
        }
        this.initKernel();
    }
    async execute(code, onExecutionPhaseChanged) {
        if (this._kernel) {
            this.clear();
            const metadata = {};
            const done = execute(this._id, code, this._outputArea, this._kernel, metadata, this._suppressCodeExecutionErrors, onExecutionPhaseChanged);
            await done;
        }
    }
    interrupt() {
        if (this._kernel) {
            this._kernel.interrupt();
        }
    }
    clear() {
        this._outputArea.model.clear();
    }
    get kernel() {
        return this._kernel;
    }
    set kernel(kernel) {
        this._kernel = kernel;
        this.initKernel();
    }
    get outputArea() {
        return this._outputArea;
    }
    initKernel() {
        if (this._kernel) {
            this._iPyWidgetsManager.registerWithKernel(this._kernel.connection);
        }
    }
}
export default OutputAdapter;
//# sourceMappingURL=OutputAdapter.js.map