import { jsxs as _jsxs } from "react/jsx-runtime";
/*
 * Copyright (c) 2021-2023 Datalayer, Inc.
 *
 * MIT License
 */
import { createContext, useContext } from 'react';
import { ServerConnection } from '@jupyterlab/services';
import { useJupyterReactStoreFromProps } from '../state';
import { requestAPI } from './JupyterHandlers';
/**
 * The instance for the Jupyter context.
 */
export const JupyterContext = createContext(undefined);
/**
 * The type for the Jupyter context consumer.
 */
export const JupyterContextConsumer = JupyterContext.Consumer;
/**
 * The type for the Jupyter context provider.
 */
const JupyterProvider = JupyterContext.Provider;
/*
 * User Jupyter hook.
 */
export const useJupyter = (props) => {
    const context = useContext(JupyterContext);
    if (context) {
        // We are within a React Context, just return the JupyterContext.
        // The provided props are irrelevant in this case.
        return context;
    }
    // We are not within a React Context....
    // so create a JupyterContext from the store based on the provided props.
    const { jupyterConfig, kernel, kernelIsLoading, serviceManager, } = useJupyterReactStoreFromProps(props ?? {});
    const storeContext = {
        defaultKernel: kernel,
        jupyterServerUrl: jupyterConfig.jupyterServerUrl,
        kernel,
        kernelIsLoading,
        kernelManager: serviceManager?.kernels,
        lite: props?.lite,
        serverless: props?.serverless ?? false,
        serverSettings: serviceManager?.serverSettings,
        serviceManager,
    };
    return storeContext;
};
/**
 * Utility method to ensure the Jupyter context
 * is authenticated with the Jupyter server.
 */
export const ensureJupyterAuth = async (serverSettings) => {
    try {
        await requestAPI(serverSettings, 'api', '');
        return true;
    }
    catch (reason) {
        console.log('The Jupyter Server API has failed with reason', reason);
        return false;
    }
};
/*
 *
 */
export const createServerSettings = (jupyterServerUrl, jupyterServerToken) => {
    return ServerConnection.makeSettings({
        baseUrl: jupyterServerUrl,
        wsUrl: jupyterServerUrl.replace(/^http/, 'ws'),
        token: jupyterServerToken,
        appendToken: true,
        init: {
            mode: 'cors',
            credentials: 'include',
            cache: 'no-store',
        },
    });
};
/**
 * The Jupyter context provider.
 */
export const JupyterContextProvider = (props) => {
    const { children, skeleton } = props;
    const { jupyterServerUrl, kernel, kernelIsLoading, lite, serverSettings, serviceManager, } = useJupyter(props);
    return (_jsxs(JupyterProvider, { value: {
            defaultKernel: kernel,
            // FIXME we should not expose sub attributes to promote single source of truth (like URLs coming from serverSettings).
            jupyterServerUrl,
            kernel,
            kernelIsLoading,
            kernelManager: serviceManager?.kernels,
            lite,
            serverSettings,
            serverless: props.serverless ?? false,
            serviceManager,
        }, children: [kernelIsLoading && skeleton, children] }));
};
//# sourceMappingURL=JupyterContext.js.map